with Text_IO, Ada.Numerics.Discrete_Random; 
use  Text_IO; 

procedure main is

	subtype Plane_Number is Positive Range 1..10;
	subtype Plane_Speed is Positive Range 1..3;
	subtype Airfloor_Number is Positive Range 1..3;
	subtype Airport_Capacity_Number is Integer Range 0..3;
	
	type Airfloor_Array is array (Airfloor_Number'Range) of Boolean;
	
	package AirfloorRandom is new Ada.Numerics.Discrete_Random (Airfloor_Number); 
	package SpeedRandom is new Ada.Numerics.Discrete_Random (Plane_Speed); 
	use AirfloorRandom, SpeedRandom; 
	
	
	-- kirst megvalst vdett objektum
	protected myWriter is 
		procedure Put (S: in String); 
	end myWriter; 
 
	protected body myWriter is 
		procedure Put(S: in string) is 
		begin 
			put_line(S); 
		end Put; 
	end myWriter;
	
	
	-- randomgenerslst megvalst vdett objektum
	protected myGenerator is 
		procedure Reset; 
		function RandomSpeed return Plane_Speed;
		function RandomFloor return Airfloor_Number;
		private 
			SG : SpeedRandom.Generator; 
			FG : AirfloorRandom.Generator;
	end myGenerator; 
 
	protected body myGenerator is
	
		Procedure Reset is 
   		begin 
			Reset(SG); 
      			Reset(FG); 
		end Reset; 
		function RandomFloor return Airfloor_Number is 
		begin 
			return Random(FG); 
		end RandomFloor; 
		function RandomSpeed return Plane_Speed is 
		begin 
			return Random(SG); 
		end RandomSpeed;
		
	end myGenerator;  
	
	
	-- irnyttornyokat megvalst vdett objektum
	protected Tower is
		-- megadja vagy megtagadja a leszllsi engedlyt
		procedure RequestLand (Floor: Airfloor_Number; B: in out Boolean);
		-- tudomsul veszi, hogy leszalltak
		procedure Boarded (Floor: Airfloor_Number);
		-- megadja vagy megtagadja a felszllsi engedlyt
		procedure RequestTakeOff (Floor: Airfloor_Number; B: in out Boolean);
		-- tudomsul veszi, hogy elhagyni kszlnek a repteret
		procedure Leaving (Floor: Airfloor_Number);
		-- tudomsul veszi, hogy elhagyjk a repteret
		procedure Leave (Floor: Airfloor_Number);
		private
			Airport_Capacity:	Airport_Capacity_Number := Airport_Capacity_Number'Last;
			Free_Airfloors:		Airfloor_Array := ( others => TRUE );
	end Tower;
	
	protected body Tower is
	
		procedure RequestLand(Floor: Airfloor_Number; B: in out Boolean) is
		begin
			if Airport_Capacity > 0 AND Free_Airfloors (Floor) then
				Airport_Capacity := Airport_Capacity - 1;
				Free_Airfloors(Floor) :=  FALSE;
				B := TRUE;
			else
				B := FALSE;
			end if;
		end RequestLand;
		
		procedure Boarded(Floor: Airfloor_Number) is
		begin
			Free_Airfloors(Floor) :=  TRUE;
		end Boarded;
		
		procedure RequestTakeOff(Floor: Airfloor_Number; B: in out Boolean) is
		begin
			B := Free_Airfloors (Floor);
		end RequestTakeOff;
		
		procedure Leaving(Floor: Airfloor_Number) is
		begin
			Free_Airfloors(Floor) :=  FALSE;
		end Leaving;
		
		procedure Leave(Floor: Airfloor_Number) is
		begin
			Airport_Capacity :=  Airport_Capacity + 1;
			Free_Airfloors(Floor) :=  TRUE;
		end Leave;
		
	end Tower;
	
	
	-- ez mondja meg, hogy mikor van vge
	task Airforce is
		Entry Left;
	end AirForce;
	
	task body AirForce is 
		db : Integer := 0; 
	begin 
		while db < Plane_Number'Last loop 
			Select 
         			Accept Left do 
					db := db + 1; 
				End Left; 
			End Select; 
		end loop; 
	end AirForce;
	
	
	-- replgpeket megvalst vdett objektum
	task type Plane (ID: Plane_Number);
	type PlanePointer is Access Plane;
	
	task body Plane is
	
		myID:		Plane_Number	:= ID;
		mySpeed:	Plane_Speed     := myGenerator.RandomSpeed;
		myFloor:	Airfloor_Number := myGenerator.RandomFloor;
		myAccess:	Boolean		:= FALSE;
		
	begin
	
		Tower.RequestLand (myFloor, myAccess);
		while ( not myAccess ) loop
			myWriter.Put (Integer'Image(myID) & ". gep koroz a torony felett");
			Delay (duration(1));
			Tower.RequestLand (myFloor, myAccess);
		end loop;
		
		myWriter.Put (Integer'Image(myID) & ". gep megkezdi a leszallast a " & Integer'Image(myFloor) & ". folyoson");
		Delay (duration(mySpeed));
		
		myWriter.Put (Integer'Image(myID) & ". leszallt");
		Tower.Boarded(myFloor);
		
		myWriter.Put (Integer'Image(myID) & ". utasokra var");
		Delay (duration(5));
		
		Tower.RequestTakeOff (myFloor, myAccess);
		while ( not myAccess ) loop
			Delay duration((1));
			Tower.RequestTakeOff (myFloor, myAccess);
		end loop;
		
		Tower.Leaving(myFloor);
		myWriter.Put (Integer'Image(myID) & ". gep megkezdi a felszallast a " & Integer'Image(myFloor) & ". folyoson");
		Delay (duration(mySpeed));
		
		Tower.Leave (myFloor);
		myWriter.Put (Integer'Image(myID) & ". felszallt");
	
		AirForce.Left;
			
	end Plane;
	 
	
	
	type PlanePointer_Array is Array (Plane_Number'Range) of PlanePointer;
	PPlane: PlanePointer_Array;
	
begin
	myGenerator.Reset;
	for I in Plane_Number'Range loop
		PPlane(I) := new Plane(I);
	end loop;
end;